/* 
   Author: Sergio Porcu

   Chinese checkers
   In this game, you can move one pawn per turn,
   along vertical or orizontal lines, "jumping over"
   another pawn and capturing it, removing it from the board.
   You can "jump over" another pawn only if there are no
   pawns in the square where the "jump" ends.
   The game ends when you cannot move pawns anymore. If there's
   only one pawn left on the board, you win. A more difficult
   objective to achieve is to win the game by placing the last pawn left
   in the square just in the middle of the board.

*/

#include <stdio.h>

/* empty place */
#define V  0
/* place with a pawn */
#define P  1
/* place outside the board */
#define I 255
/* board size */
#define SIZE 8

int muovi (int scacc[SIZE+1][SIZE+1], int mosse[31][4], int n);

main ()
{
  /* Board */
  int scacc[SIZE+1][SIZE+1] =
    {
      {I, I, I, I, I, I, I, I, I},
      {I, I, I, P, P, P, I, I, I},
      {I, I, I, P, P, P, I, I, I},
      {I, P, P, P, P, P, P, P, I},
      {I, P, P, P, V, P, P, P, I},
      {I, P, P, P, P, P, P, P, I},
      {I, I, I, P, P, P, I, I, I},
      {I, I, I, P, P, P, I, I, I},
      {I, I, I, I, I, I, I, I, I}
    };

  /* Buffer where moves are stored */
  int mosse[31][4];

  FILE  *fout;
  int i;
  char nomeFile[80];
 
  if (muovi (scacc, mosse, 0) == 0) {
    fprintf (stdout, "A viable solution does not exist.\n");
    return;
  }
 
  fprintf (stdout, "Output filename: ");
  scanf ("%s", nomeFile);
  fout = fopen (nomeFile, "w");
  if (fout == NULL) {
    fprintf (stderr, "Error while opening file %s\n", nomeFile);
    return;
  }
 
  for (i=0; i<31; i++) {
   fprintf (fout, "%d %d %d %d\n", mosse[i][0], mosse[i][1],
     mosse[i][2], mosse[i][3]);
  }
 
  fclose (fout);

  return;
}

/*
 *  Recursive function
 */

int
muovi (
  int scacc[SIZE+1][SIZE+1],
  int mosse[31][4],
  int n
  )
{
  /* Offset of the pawn to be captured */
  int xOffPedina[4] = { 0, 1, 0, -1};
  int yOffPedina[4] = { -1, 0, 1, 0};
  /* Offset of the end place of the jump */
  int xOffArrivo[4] = { 0, 2, 0, -2};
  int yOffArrivo[4] = { -2, 0, 2, 0};

  int x1, y1, x2, y2, x3, y3, k;

#if 0
  /* Debug */
  printf ("%d\n", n);
#endif

  /* End condition */
  if (n == 31) {
    return (1);
  }

  /* For every square on the board ... */
  for (x1=1; x1<SIZE; x1++) {
    for (y1=1; y1<SIZE; y1++) {
      /* ... if there's a pawn ... */
      if (scacc[x1][y1] == P) {
        /* ... if it can be captured ... */
        for (k=0; k<4; k++) {
          x2 = x1 + xOffPedina[k];
          y2 = y1 + yOffPedina[k];
          if (scacc[x2][y2] == P) {
            x3 = x1 + xOffArrivo[k];
            y3 = y1 + yOffArrivo[k];
            if (scacc[x3][y3] == V) {
              /* ... moving a pawn ... */
              scacc[x3][y3] = P;
              scacc[x2][y2] = V;
              scacc[x1][y1] = V;
              /* ... recursion ... */
              if (muovi (scacc, mosse, n+1) == 1) {
                /* ... if the move was OK, it's stored ... */
                mosse[n][0] = x1;
                mosse[n][1] = y1;
                mosse[n][2] = x3;
                mosse[n][3] = y3;
                return (1);
              } else {
                /* ... otherwise, a backtrack is executed */
                scacc[x3][y3] = V;
                scacc[x2][y2] = P;
                scacc[x1][y1] = P;
              }
            }
          }
        }
      }
    }
  }

  return (0);
}

