/* 
   Author: Sergio Porcu

   
Tower of Hanoi 
Recursive and Iterative solutions
There are only two rules in this game:
1) you can move only the disk placed on the top of a tower
2) you cannot place a certain disk on a disk smaller than it.

The objective of the game is to move all the disks from one stick
to another one, placing them in the same original order.

*/

#include <stdio.h>

void hanoiRic (int *, int, int, int);
void hanoiIte (int);

main ()
{
  int mossa, n;

  fprintf (stdout, "Numero of disks: ");
  scanf ( "%d", &n);

  mossa = 1;
  fprintf (stdout, "HANOI Recursive:\n");
  hanoiRic (&mossa, n, 0, 2);

  fprintf (stdout, "HANOI Iterative:\n");
  hanoiIte (n);

  return;
}

void
hanoiRic (
  int *mossa,
  int n,
  int source,
  int dest
  )
{
  int aux;

  if (n > 0) {
    aux = 3 - (source + dest);
    hanoiRic (mossa, n - 1, source, aux);
    fprintf (stdout,  "(%d) Source %d - Dest. %d \n", *mossa, source, dest);
    *mossa = *mossa + 1;
    hanoiRic (mossa, n - 1, aux, dest);
  }

  return;
}

void
hanoiIte (
  int n
  )
{
  int *pos, numeroMosse;
  int i, disc, mossa, delta, newPos;
 
  /* dynamic vector, containing the position of the disks on the sticks (0, 1, 2) */
  pos = (int *) malloc (n * sizeof (int));

  /* number of moves */
  numeroMosse = (1<<n)-1;

  /* Initializing: all disks are placed on the stick 0 */
  for (i=0;i<n;i++) {
    pos[i]=0;
  }

  for (mossa=0; mossa<numeroMosse; mossa++) {
    /* Searching the disk to move: delta is the movement */
    for (disc=0, delta=n&1; disc<n; disc++, delta^=1) {
      if (((mossa >> disc)&1)==0) {
        break;
      }
    }

    /* New disk position */
    newPos = (pos[disc]+1+delta)%3;

    fprintf (stdout,  "(%d) Source %d - Dest. %d \n", mossa+1,
      pos[disc], newPos);

    pos[disc]=newPos;
  }

  free (pos);

  return;
}





